/* (c) 1985, Phoenix Computer Products Corp. and Novum Organum, Inc. */
/* Copyright (c) Novum Organum, Inc.  1985
*
* name:		SyTone    -- make the speaker speak
*		StTune    -- sound out a sequence of freq./duration pairs
*		SyBeep    -- sound out the Novum Organum tune
*		SySetBeep -- set your own tune for SyBeep to play
*
* synopsis:	void	SyTone(), SyTune(), SyBeep(), SySetBeep();
*		int	freq;		/+ frequency of tone in Hz +/
*		int	time;		/+ duration of tone in centiseconds +/
*		int	*freqs;		/+ -1 terminated list of frequencies +/
*		int	*times;		/+ -1 terminated list of durations +/
*
*		SyTone( freq, time );
*		SyTune( freqs, times );
*		SyBeep();
*		SySetBeep( freqs, times );
*
*
* description:	SyTone() causes speaker to produce a tone of a specified
*		frequency and duration.  A frequency of zero is taken to
*		mean silence of the specified duration (this may be useful
*		in conjunction with SyTune(), or to produce accurate
*		program delays).
*		SyTune() calls SyTone() for each of a -1 terminated
*		list of frequencies and durations.  If the duration list
*		runs out before the frequency list, it is reused.
*		SyBeep() calls SyTune() to play a standard tune.
*		SySetBeep() allows you to replace the standard tune.
*		Pointers to the new standard tune (not copies) are saved, so
*		the arguments to SySetBeep() must be permanent storage (static
*		or heap), NOT automatic.
*
*		All times are in centiseconds; all frequencies are in Hz.
*		When you forget to -1 terminate your tune list, you'll
*		probably get a very long bizarre tune.  When you get a very
*		long bizarre tune, please remember to terminate your list.
*     
* returns:	nothing, nothing, nothing, nothing
*
***/

#include "cptpdf.h"
#include "psys.h"

/* There are two methods of counting out the duration of a tone:
 * counting timer cycles, and counting time-of-day clock ticks.
 * The former method has a higher resolution, but due to hardware
 * incompatibility on some clones, is less reliable.  If the flag
 * which follows this comment is turned on (made non-zero), we'll
 * try to detect whether the cycle-count method will work at
 * run-time, and use it if possible.  It has been reported to us,
 * however, that our test is not 100% reliable:  it will fail on
 * a certain yet-smaller number of clones.  If the flag is turned
 * off, we'll always use the time-of-day clock.  This provides the
 * greatest reliability, but guaranteed poor (~.05sec) resolution.
 */
#define	TRY_COUNT	0

#ifdef _Phoenix

static	int	_ourfreq[] = { 880, -1 };
static	int	_ourtime[] = {  16, -1 };
static	int	*_stdfreq  = _ourfreq;
static	int	*_stdtime  = _ourtime;

#endif


/*VOID*/sytone	( freq, time )
	int	freq, time;
{
	int	oldval, divisor, mask;
	long	wait;

#if TRY_COUNT
	long	count;
static	int	been_here = NO;
static	int	do_counts = NO;

/* First time through, check to see that we can read timer 2's output
   in the sound port.  Certain clones are hardware incompatible with
   the IBM machines, and we'll have to call sywait() instead of counting
   timer periods.  The advantage of counting is that short intervals are
   possible (sywait has a only a 5 csec resolution).
 */
	if	( !been_here )
		{
		been_here = YES;
		oldval = _portinp( 0x61 ) & 0xFC;
		_portout( 0x43, 0xB6 );
		_portout( 0x42, 0x02 );
		_portout( 0x42, 0x00 );
		_portout( 0x61, oldval | 0x01 );
		count = 1000;	/* any large number will do */
		while	( count-- )
			if	( (oldval^_portinp(0x61)) & 0x20 )
				{
				do_counts = YES;
				break;	
				}
		}
#endif

	mask = 0x03;		/* value to produce sound using timer */

#ifdef _Phoenix

	if	( freq == 0 )
		{
		freq = 1000;	/* any large number would do */
		mask = 0x01;	/* value for no sound -- just timer ticks */
		}
#endif

				/* timer can't produce frequencies < 21Hz */
				/* use minimum frequency if freq < 21 */
	divisor = ( freq < 21 ) ? 0xFFFF : (int)( 1193000L / freq );

	_portout ( 0x43, 0xB6 );		/* set timer mode */
	_portout ( 0x42, LOBYTE(divisor) );	/* set timer divisor */
	_portout ( 0x42, HIBYTE(divisor) );

	oldval = _portinp( 0x61 ) & 0xFC;
	_portout ( 0x61, oldval | mask );	/* enable spkr in timer mode */

#if TRY_COUNT
	if	( do_counts )
		{
						/* number of timer periods */
		count = ((long)freq * (long)time + 50L) / 100L;
		while	( count-- > 0 )
			{
			while(  (_portinp(0x61) & 0x20) ) {}
			while( !(_portinp(0x61) & 0x20) ) {}
			}
		}
	else
#endif
//		  sywait( time );

	wait = (long) time * 10L;
	WAITMSL(wait);

	_portout ( 0x61, oldval );		/* turn off speaker */
}


#ifdef _Phoenix

/*VOID*/sytune	( freqs, times )
	int	*freqs;
	int	*times;
{
	int	freq, time;
	int	*ttt;

	ttt = times;
	while ( (freq = *freqs++) != -1 )
		{
		time = *ttt++;
		if	( time == -1 )
			{
			ttt = times;
			time = *ttt++;
			}
		sytone( freq, time );
		}
}

#endif


/*VOID*/sybeep()
{
#ifdef _Phoenix
	sytune( _stdfreq, _stdtime );
#else
	sytone( 880, 16);
#endif
}


#ifdef _Phoenix

/*VOID*/sysetbeep( freqs, times )
	int	*freqs;
	int	*times;
{
	if	(freqs)
		{
		_stdfreq = freqs;
		_stdtime = times;
		}
	else
		{
		_stdfreq  = _ourfreq;
		_stdtime  = _ourtime;
		}
}

#endif
